==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/special.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: special.c,v 1.9 2003/10/08 17:32:10 lwa Exp $";

#include "config.h"

#include <sys/types.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>

#include "lmtp.h"

#define skipblanks(xbuf) while(isblank(*(xbuf))) (xbuf)++

RE *special_return_path(struct fstate* state) {
  RE *re;
  char *path = state->sender;
  if (path == NULL || *path == 0) {
    path = "MAILER-DAEMON";
  }
  re = re_new(path, "s");
  return re;
}

RE *special_mailinglists(struct fstate* state) {
  RE *re = NULL;
  char *file = state->mailinglists_file;
  if (file) {
    int fd = open(state->mailinglists_file, O_RDONLY);
    if (fd>=0) {
      struct stat sb;
      if (fstat(fd, &sb) == 0) {
	ssize_t size = sb.st_size;
	char *list = malloc(size+1);
	if (list) {
	  size=read(fd, list, size);
	  if (size>0) {
	    char *a = list;
	    list[size]=0;
	    while(*a) { /* remplace '\n' by ',' */
	      if (*a == '\n') 
		*a = ',';
	      a++;
	    }
	    re = re_new(list, "a");
	  }
	}
	free(list);
      }
      close(fd);
    } else {
      if (errno == ENOENT)
	re = re_new(NULL, "0"); /* false */
    }
  } else {
    re = re_new(NULL, "0"); /* false */
  }
  if (re == NULL) 
    re = re_new(NULL, ""); /* tmp error */
  return re;
}

RE *special_aliases(struct fstate* state) {
  RE *re = NULL;
  int off = state->alias_sub_addresses;

  static regmatch_t *pmatch = NULL;

  if (pmatch == NULL) {
    pmatch = malloc(sizeof(regmatch_t) * (off+2));
    if (pmatch == NULL)
      return re_new(NULL, ""); /* tmp error */
  }

  if (state->alias_file &&
      state->alias_format &&
      state->alias_sub_addresses >= 0) {
    int fd = open(state->alias_file, O_RDONLY);

    if (fd>=0) {
      struct stat sb;
      if (fstat(fd, &sb) == 0) {
	ssize_t size = sb.st_size;
	char *list = malloc(size+1);
	char *addresses = malloc(size+1);
	if (list && addresses) {
	  size=read(fd, list, size);
	  if (size>0) {
	    char *addresses_c = addresses;
	    char *line = list;
	    list[size]=0;
	    while(line) {
	      char *end;
	      if ((end= strchr(line, '\n')) != NULL) 
		*end++ = 0;
	      if (*line &&
		  regexec(state->alias_format, line, off+1, pmatch, 0) == 0) {
		memcpy(addresses_c, line + pmatch[off].rm_so,
		       pmatch[off].rm_eo - pmatch[off].rm_so);
		addresses_c += pmatch[off].rm_eo - pmatch[off].rm_so;
		*addresses_c++ = ',';
	      }
	      line = end;
	    }
	    *addresses_c = 0;
	    /* replace ' ' by ',' for .mailrc alias compatibility */
	    addresses_c = addresses;
	    while(*addresses_c) {
	      if (*addresses_c == ' ' || *addresses_c == '\t')
		*addresses_c = ' ';
	      addresses_c++;
	    }
	    re = re_new(addresses, "a");
	    fprintf(stderr, "add=%s\n", addresses);
	  }
	}
	free(list);
	free(addresses);
      }
      close(fd);
    } else { /* fd == -1 */
      if (errno == ENOENT)
	re = re_new(NULL, "0"); /* false */
    }
  } else { /* not configured */
    re = re_new(NULL, "0"); /* false */
  }
  if (re == NULL) 
    re = re_new(NULL, ""); /* tmp error */
  return re;
}

struct special_t { 
  char *name;
  RE *(*f)(struct fstate*);
} specials [] = {
  { "return-path", special_return_path },
  { "mailinglists", special_mailinglists },
  { "alias", special_aliases },
  { NULL, NULL }
};

RE *special_re(struct fstate *state, char **buf) {
  char *xbuf_b = *buf+1; /* skip $ */
  char *xbuf_e = xbuf_b;
  struct special_t *special = specials;
  while(*xbuf_e && !isblank(*xbuf_e))  xbuf_e ++;
  while(special->name) {
    if (strncasecmp(special->name, xbuf_b, xbuf_e - xbuf_b) == 0) {
      *buf = xbuf_e;
      return special->f(state);
    }
    special ++;
  }
  return NULL;
}

==> lmtpd-0.10.4.tar.gz: lmtpd-0.10.4/special.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: special.c,v 1.10 2004/10/26 09:59:20 lwa Exp $";



#include <sys/types.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>

#include "lmtp.h"

#define skipblanks(xbuf) while(isblank(*(xbuf))) (xbuf)++

RE *special_return_path(struct fstate* state) {
  RE *re;
  char *path = state->sender;
  if (path == NULL || *path == 0) {
    path = "MAILER-DAEMON";
  }
  re = re_new(path, "s");
  return re;
}

RE *special_mailinglists(struct fstate* state) {
  RE *re = NULL;
  char *file = state->mailinglists_file;
  if (file) {
    int fd = open(state->mailinglists_file, O_RDONLY);
    if (fd>=0) {
      struct stat sb;
      if (fstat(fd, &sb) == 0) {
	ssize_t size = sb.st_size;
	char *list = malloc(size+1);
	if (list) {
	  size=read(fd, list, size);
	  if (size>0) {
	    char *a = list;
	    list[size]=0;
	    while(*a) { /* remplace '\n' by ',' */
	      if (*a == '\n') 
		*a = ',';
	      a++;
	    }
	    re = re_new(list, "a");
	  }
	}
	free(list);
      }
      close(fd);
    } else {
      if (errno == ENOENT)
	re = re_new(NULL, "0"); /* false */
    }
  } else {
    re = re_new(NULL, "0"); /* false */
  }
  if (re == NULL) 
    re = re_new(NULL, ""); /* tmp error */
  return re;
}

RE *special_aliases(struct fstate* state) {
  RE *re = NULL;
  int off = state->alias_sub_addresses;

  static regmatch_t *pmatch = NULL;

  if (pmatch == NULL) {
    pmatch = malloc(sizeof(regmatch_t) * (off+2));
    if (pmatch == NULL)
      return re_new(NULL, ""); /* tmp error */
  }

  if (state->alias_file &&
      state->alias_format &&
      state->alias_sub_addresses >= 0) {
    int fd = open(state->alias_file, O_RDONLY);

    if (fd>=0) {
      struct stat sb;
      if (fstat(fd, &sb) == 0) {
	ssize_t size = sb.st_size;
	char *list = malloc(size+1);
	char *addresses = malloc(size+1);
	if (list && addresses) {
	  size=read(fd, list, size);
	  if (size>0) {
	    char *addresses_c = addresses;
	    char *line = list;
	    list[size]=0;
	    while(line) {
	      char *end;
	      if ((end= strchr(line, '\n')) != NULL) 
		*end++ = 0;
	      if (*line &&
		  regexec(state->alias_format, line, off+1, pmatch, 0) == 0) {
		memcpy(addresses_c, line + pmatch[off].rm_so,
		       pmatch[off].rm_eo - pmatch[off].rm_so);
		addresses_c += pmatch[off].rm_eo - pmatch[off].rm_so;
		*addresses_c++ = ',';
	      }
	      line = end;
	    }
	    *addresses_c = 0;
	    /* replace ' ' by ',' for .mailrc alias compatibility */
	    addresses_c = addresses;
	    while(*addresses_c) {
	      if (*addresses_c == ' ' || *addresses_c == '\t')
		*addresses_c = ' ';
	      addresses_c++;
	    }
	    re = re_new(addresses, "a");
	    fprintf(stderr, "add=%s\n", addresses);
	  }
	}
	free(list);
	free(addresses);
      }
      close(fd);
    } else { /* fd == -1 */
      if (errno == ENOENT)
	re = re_new(NULL, "0"); /* false */
    }
  } else { /* not configured */
    re = re_new(NULL, "0"); /* false */
  }
  if (re == NULL) 
    re = re_new(NULL, ""); /* tmp error */
  return re;
}

struct special_t { 
  char *name;
  RE *(*f)(struct fstate*);
} specials [] = {
  { "return-path", special_return_path },
  { "mailinglists", special_mailinglists },
  { "alias", special_aliases },
  { NULL, NULL }
};

RE *special_re(struct fstate *state, char **buf) {
  char *xbuf_b = *buf+1; /* skip $ */
  char *xbuf_e = xbuf_b;
  struct special_t *special = specials;
  while(*xbuf_e && !isblank(*xbuf_e))  xbuf_e ++;
  while(special->name) {
    if (strncasecmp(special->name, xbuf_b, xbuf_e - xbuf_b) == 0) {
      *buf = xbuf_e;
      return special->f(state);
    }
    special ++;
  }
  return NULL;
}
